/*------------------------------------------------------------------------------*
 * Architecture & Implementation of DBMS                                        *
 *------------------------------------------------------------------------------*
 * Copyright 2019 Databases and Information Systems Group TU Dortmund          *
 *                                                                              *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS      *
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,  *
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL      *
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR         *
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR        *
 * OTHER DEALINGS IN THE SOFTWARE.                                              *
 *------------------------------------------------------------------------------*
 */

#pragma once
#include <cstdint>
#include <chrono>
#include <iostream>

namespace aidbms::base {
    class Clock
    {
    public:
        Clock(const char *name = nullptr, const std::size_t operations = 0) : _name(name), _operations(operations) {
            _start = std::chrono::steady_clock::now();
        }

        ~Clock() {
            const auto end = std::chrono::steady_clock::now();
            const auto milliseconds = std::chrono::duration_cast<std::chrono::milliseconds>(end - _start);

            if(_name) {
                std::cout << _name << ": ";
            }
            std::cout << milliseconds.count() << " ms.";

            if(_name && _operations) {
                const auto seconds = milliseconds.count() / 1000.0;
                const auto throughput = double(_operations) / seconds;
                std::cout << " (" << throughput << " " << _name << "/second)";
            }

            std::cout << std::endl;
        }

    private:
        const char *_name;
        const std::size_t _operations;
        std::chrono::time_point<std::chrono::steady_clock> _start;
    };
}