/*------------------------------------------------------------------------------*
 * Architecture & Implementation of DBMS                                        *
 *------------------------------------------------------------------------------*
 * Copyright 2019 Databases and Information Systems Group TU Dortmund          *
 *                                                                              *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS      *
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,  *
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL      *
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR         *
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR        *
 * OTHER DEALINGS IN THE SOFTWARE.                                              *
 *------------------------------------------------------------------------------*
 */

#pragma once

#include <cstdint>
#include <benchmarks/workload.h>
#include "compression/wah_bit_vector.h"

namespace aidbms::benchmarks {
    class BitStream : public Workload<bool>
    {
    public:
        /**
         * @brief CompressionWorkload reads, parses the file at the specified path and generates an compression
         * workload.
         * @param workload_file_name path to file
         */
        BitStream(const char *workload_file_name);

        BitStream() {

        }

        static std::vector<aidbms::compression::WAHBitVector> read_compressed_bitstream(const char *workload_file_name);

        const bool operator[] (const std::size_t index) const {
            return _items[index];
        }
    };
}
