/*------------------------------------------------------------------------------*
 * Architecture & Implementation of DBMS                                        *
 *------------------------------------------------------------------------------*
 * Copyright 2019 Databases and Information Systems Group TU Dortmund          *
 *                                                                              *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS      *
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,  *
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL      *
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR         *
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR        *
 * OTHER DEALINGS IN THE SOFTWARE.                                              *
 *------------------------------------------------------------------------------*
 */

#include <benchmarks/bit_stream.h>
#include <string>
#include <regex>
#include <cstdlib>
#include <random>
#include <iostream>

using namespace aidbms::benchmarks;

BitStream::BitStream(const char *workload_file_name)
{
    std::string line;
    std::ifstream file(workload_file_name);

    while(std::getline(file, line)) {
        for(const auto item : line) {
            if(item == '0' || item == '1') {
                this->add(item == '1');
            }
        }
    }
}

std::vector<aidbms::compression::WAHBitVector> BitStream::read_compressed_bitstream(const char *bitstream_file_name)
{
    std::vector<aidbms::compression::WAHBitVector> compressed;
    std::string line;
    std::ifstream file(bitstream_file_name);
    aidbms::compression::WAHBitVector bitvector;

    while(std::getline(file, line)) {
        int i = 0;
        bitvector.is_fill(line[0] == '0' ? 0 : 1);
        for(const auto item : line){
            if(item == '0') {
                bitvector.set(31 - i, 0);
            }
            else if (item == '1') {
                bitvector.set(31 - i, 1);
            }
            i++;
        }
        compressed.push_back(bitvector);
    }
    return compressed;
}

